
#property description "Summarizes ask and bid activity"
#property description " "

#property copyright "2015 Ovo"
#property version "101.000"
#property link "ovo.cz"

#property icon "\\Images\\avatar64.ico";

#property indicator_separate_window
#property indicator_buffers 3

#property indicator_color1 clrLightSeaGreen
#property indicator_color2 clrRed
#property indicator_color3 clrGray
#property indicator_style3 STYLE_DOT

//--- indicator parameters

enum REVERSE_ENUM { 
   DRAW_NORMAL, // In Line
   DRAW_REVERSE  // Oposite
};
enum DISPLAY_ENUM { 
   _DRAW_HISTOGRAM = DRAW_HISTOGRAM, // Histogram
   _DRAW_LINE = DRAW_LINE // Line
};

input DISPLAY_ENUM DRAW_DISPL = _DRAW_HISTOGRAM; // Chart Display
/*input*/ REVERSE_ENUM DRAW_DIRECTION = DRAW_REVERSE; // Sellers Display

#include <Ovo/MT4IndicatorManager.class.mqh>
#include <Ovo/MT4DrawingIndicator.class.mqh>
#include <Ovo/winapi/MT4WinapiFile.class.mqh>

void main() { 
   if (!IsDllsAllowed()) {
      Alert("Please allow DLL imports in the Common tab of the indicator settings.");
      manager.initResult = INIT_FAILED;
   } else {
      IndicatorDigits(0);
      
      if (DRAW_DIRECTION == DRAW_REVERSE) {
         IndicatorSetInteger(INDICATOR_LEVELS, 1);
         IndicatorSetDouble(INDICATOR_LEVELVALUE, 1, 0.0);         
      }
      
      MT4Scope* scope = manager.destroyOnExit(new MT4Scope());
      manager.add(new PriceTickCounter(scope));
   }
}

struct PriceTickStructure:MT4Structure {
   datetime time;
   uint bidCounter;
   uint askCounter;
};

class PriceTickCounter: public MT4DrawingIndicator {

   //--- indicator parameters
   
   //--- buffers
   
   MT4DrawingBuffer* sellerCounterBuffer;
   MT4DrawingBuffer* buyerCounterBuffer;
   MT4DrawingBuffer* difCounterBuffer;
   
   MT4File* indicatorFile;
   MT4File* controlFile;
   bool hasControl;
   PriceTickStructure record;
   
   MqlTick recentTick;
   
   string server;
   
   
   
   #define SellerCounterBuffer sellerCounterBuffer.value
   #define BuyerCounterBuffer buyerCounterBuffer.value
   #define DifCounterBuffer difCounterBuffer.value
   
public:
   PriceTickCounter(MT4Scope* aScope , bool calculationOnly=false)
   : MT4DrawingIndicator(aScope, calculationOnly) {
      difCounterBuffer = addDrawingBuffer("Dif");
      hasControl = false;      
      record.askCounter = 0;
      record.bidCounter = 0;
      server = "";
   }

   ~PriceTickCounter() {
      deleteSafely(indicatorFile);
      deleteSafely(controlFile);
   }
   
   void init() {
      //--- check for input parameter
      sellerCounterBuffer = addDrawingBuffer("Bid Rate");
      buyerCounterBuffer = addDrawingBuffer("Ask Rate");
      sellerCounterBuffer.setType(DRAW_DISPL);
      buyerCounterBuffer.setType(DRAW_DISPL);
      
   }
   
   void checkServer() {
      string newServer = AccountInfoString(ACCOUNT_SERVER);
      if (newServer == "") {
         MT4WinapiFile f("");
         newServer = f.setHistoryPath();
      }
      if (newServer != server) {
         server = newServer;
         deleteSafely(indicatorFile);
         deleteSafely(controlFile);         
         // set the file for recording, if possible, open it RW, otherwise RO
         MT4String path;
         path + TerminalInfoString(TERMINAL_COMMONDATA_PATH) ;      
         MT4WinapiFile dir("", path.toString());

         dir.createDirectory(server);
         path + "\\" + server;
         MT4String name;
         name + symbol + "_askbidcounter.data";
         indicatorFile = new MT4WinapiFile(name.toString(), path.toString());
         controlFile = new MT4WinapiFile(name.toString(), path.toString());
         takeControl();
         indicatorFile.openRO();
      }
   }
   
   void calculate() {
      resizeBuffers();
      setBuffersAsSeries(true);
      
     checkServer();
     if (!hasControl) {
         takeControl();
      }
      if (hasControl) {
         updateFile();
      }

      // re-read the last bar
      int newBars = indicatorAdded(true);
      bool isEnd = false;

      PriceTickStructure pts;
      indicatorFile.seekFromEnd(sizeof(pts));
      if (indicatorFile.readStruct(pts, sizeof(pts)) > 0) {

         for (int i = 0; i < newBars && !isEnd; i++) {
            
            datetime nexttime = iTime(symbol, timeframe, i);
            BuyerCounterBuffer[i]=0;
            SellerCounterBuffer[i]=0;
            DifCounterBuffer[i]=0;
            
            while (pts.time >= nexttime) {
               BuyerCounterBuffer[i] += pts.askCounter;
               DifCounterBuffer[i] += pts.askCounter;
               
               if (DRAW_DIRECTION==DRAW_NORMAL) {
                  SellerCounterBuffer[i] += pts.bidCounter;
               }
               else {
                  SellerCounterBuffer[i] -= pts.bidCounter;
               }
               DifCounterBuffer[i] -= pts.bidCounter;
               
               // seek next
               if (!indicatorFile.seekFromCurrent(-2 * sizeof(PriceTickStructure))) {
                  isEnd = true;
                  break;                     
               }
               indicatorFile.readStruct(pts, sizeof(pts));
               // stop on file end
               
            }
            // replace zero by nothing
            if (BuyerCounterBuffer[i]==0 && SellerCounterBuffer[i]==0) {
               BuyerCounterBuffer[i] = EMPTY_VALUE;
               SellerCounterBuffer[i]= EMPTY_VALUE;
            }
         }
      }
      else {
         ERROR("Error reading file.");
      }
   }
   
   void takeControl() {
      if (controlFile.openRW()) {
         hasControl = true;
         if (controlFile.fileSize() == 0) {
            record.askCounter = 0;
            record.bidCounter = 0;
            record.time = TimeCurrent() / 60 * 60;
         } else {
            controlFile.seekFromEnd(sizeof(record));
            controlFile.read(record, sizeof(record));
         }
         INFO("Got control of file " + controlFile.getFullName());
      }    
   }
   
   void updateFile() {
      MqlTick currentTick;
      if(SymbolInfoTick(symbol,currentTick)) {
         // check time
         if (currentTick.time >= record.time + 60) {
            // append record
            controlFile.seekFromEnd(0);
            // initialize record
            record.time = currentTick.time /60 *60;
            record.askCounter = 0;
            record.bidCounter = 0;
         } else {
            // rewrite record
            controlFile.seekFromEnd(sizeof(record));
         }
         
         /* in the direction
         if (currentTick.ask > recentTick.ask) {
            record.askCounter++;
         }
         if (currentTick.bid < recentTick.bid) {
            record.bidCounter++;
         }
         */
         // just count change
         if (currentTick.ask != recentTick.ask) {
            record.askCounter++;
         }
         if (currentTick.bid != recentTick.bid) {
            record.bidCounter++;
         }
         
         
      }
      controlFile.writeStruct(record, sizeof(record));
      controlFile.flush();
      recentTick = currentTick;
     
   }
};